@@ -4,11 +4,18 @@ import os
 import re
 from dataclasses import dataclass
 from pathlib import Path, PurePath
-from typing import Dict, List, Optional, Set, Tuple
+from typing import Union
 from uuid import uuid4
 
 from robot.parsing.lexer.tokens import Token
-from robot.parsing.model.blocks import File, SettingSection, TestCase, TestCaseSection
+from robot.parsing.model.blocks import (
+    File,
+    Keyword,
+    KeywordSection,
+    SettingSection,
+    TestCase,
+    TestCaseSection,
+)
 from robot.parsing.model.statements import (
     Comment,
     EmptyLine,
@@ -18,8 +25,10 @@ from robot.parsing.model.statements import (
     Metadata,
     ResourceImport,
     SectionHeader,
+    Setup,
     Statement,
     Tags,
+    Teardown,
     TestCaseName,
     VariablesImport,
 )
@@ -31,6 +40,7 @@ from .model import (
     InteractionDetails,
     InteractionType,
     ParameterUseType,
+    SequencePhase,
     TestCaseDetails,
     TestStructureTreeNode,
     UdfType,
@@ -42,7 +52,7 @@ SEPARATOR = "    "
 ROBOT_PATH_SEPARATOR = "/"
 RELATIVE_RESOURCE_INDICATOR = r"^{root}"
 SECTION_SEPARATOR = [EmptyLine.from_params()] * 2
-TESTCASE_SEPARATOR = [EmptyLine.from_params()]
+LINE_SEPARATOR = [EmptyLine.from_params()]
 UNKNOWN_IMPORT_TYPE = str(uuid4())
 LIBRARY_IMPORT_TYPE = str(uuid4())
 RESOURCE_IMPORT_TYPE = str(uuid4())
@@ -55,47 +65,49 @@ class InteractionCall:
 
 @dataclass
 class AtomicInteractionCall(InteractionCall):
-    cbv_parameters: Dict[str, str]
-    cbr_parameters: Dict[str, str]
+    cbv_parameters: dict[str, str]
+    cbr_parameters: dict[str, str]
     indent: int
     import_prefix: str
+    sequence_phase: str
 
 
 @dataclass
 class CompoundInteractionCall(InteractionCall):
-    cbv_parameters: Dict[str, str]
-    cbr_parameters: Dict[str, str]
+    cbv_parameters: dict[str, str]
+    cbr_parameters: dict[str, str]
     indent: int
+    sequence_phase: str
 
 
 class RfTestCase:
     def __init__(self, test_case_details: TestCaseDetails, config: Configuration) -> None:
         self.uid: str = test_case_details.uniqueID
-        self.interaction_calls: List[InteractionCall] = []
-        self.used_imports: Dict[str, Set[str]] = {}
+        self.interaction_calls: list[InteractionCall] = []
+        self.used_imports: dict[str, set[str]] = {}
         self.config = config
         self.lib_pattern_list = [re.compile(pattern) for pattern in config.rfLibraryRegex]
         self.res_pattern_list = [re.compile(pattern) for pattern in config.rfResourceRegex]
         for interaction in test_case_details.interactions:
             self._get_interaction_calls(interaction)
         self.rf_tags = self._get_tags(test_case_details)
+        self.setup_keyword: Keyword = None
+        self.teardown_keyword: Keyword = None
         # TODO description
 
     @staticmethod
-    def _get_tags(test_case_details: TestCaseDetails) -> List[str]:
+    def _get_tags(test_case_details: TestCaseDetails) -> list[str]:
         tags = [keyword.name for keyword in test_case_details.spec.keywords]
         tags.extend([udf.robot_tag for udf in test_case_details.spec.udfs if udf.robot_tag])
         return tags
 
     @staticmethod
-    def _get_udf_tags(user_defined_fields: List[UserDefinedField]) -> List[str]:
+    def _get_udf_tags(user_defined_fields: list[UserDefinedField]) -> list[str]:
         udfs = []
         for udf in user_defined_fields:
-            if udf.valueType == UdfType.Enumeration:
+            if udf.valueType in [UdfType.Enumeration, UdfType.String] and udf.value:
                 udfs.append(f"{udf.name}:{udf.value}")
-            elif udf.valueType == UdfType.String and udf.value:
-                udfs.append(f"{udf.name}:{udf.value}")
-            elif udf.valueType == UdfType.Boolean and udf.value == "true":
+            elif udf.valueType == UdfType.Boolean and udf.value.lower() == "true":
                 udfs.append(udf.name)
         return udfs
 
@@ -119,8 +131,8 @@ class RfTestCase:
 
     def _append_atomic_ia(
         self,
-        cbr_params: Dict[str, str],
-        cbv_params: Dict[str, str],
+        cbr_params: dict[str, str],
+        cbv_params: dict[str, str],
         indent: int,
         interaction: InteractionDetails,
     ):
@@ -137,10 +149,11 @@ class RfTestCase:
                 cbr_parameters=cbr_params,
                 indent=indent,
                 import_prefix=import_prefix,
+                sequence_phase=interaction.spec.sequencePhase,
             )
         )
 
-    def _get_keyword_import(self, interaction) -> Tuple[str, str]:
+    def _get_keyword_import(self, interaction) -> tuple[str, str]:
         for pattern in self.lib_pattern_list:
             match = pattern.search(interaction.path)
             if match:
@@ -163,8 +176,8 @@ class RfTestCase:
 
     def _append_compound_ia_and_analyze_children(
         self,
-        cbr_params: Dict[str, str],
-        cbv_params: Dict[str, str],
+        cbr_params: dict[str, str],
+        cbv_params: dict[str, str],
         indent: int,
         interaction_detail: InteractionDetails,
     ):
@@ -174,23 +187,26 @@ class RfTestCase:
                 cbv_parameters=cbv_params,
                 cbr_parameters=cbr_params,
                 indent=indent,
+                sequence_phase=interaction_detail.spec.sequencePhase,
             )
         )
         for interaction in interaction_detail.interactions:
             self._get_interaction_calls(interaction, indent)
 
-    def _create_rf_keywords(
-        self, interaction_calls: List[InteractionCall]
-    ) -> List[List[Statement]]:
-        keyword_lists: List[List[Statement]] = [[]]
+    def _create_rf_keyword_calls(
+        self, interaction_calls: list[InteractionCall]
+    ) -> list[list[Statement]]:
+        keyword_lists: list[list[Statement]] = [[]]
         tc_index = 0
         is_first_atomic = True
         for interaction_call in interaction_calls:
             if isinstance(interaction_call, AtomicInteractionCall):
-                if self.is_splitting_ia(interaction_call, keyword_lists, tc_index):
-                    if not is_first_atomic:
-                        tc_index += 1
-                        keyword_lists.append([])
+                if (
+                    self.is_splitting_ia(interaction_call, keyword_lists, tc_index)
+                    and not is_first_atomic
+                ):
+                    tc_index += 1
+                    keyword_lists.append([])
                 is_first_atomic = False
                 keyword_lists[tc_index].append(self._create_rf_keyword(interaction_call))
             elif (
@@ -207,21 +223,96 @@ class RfTestCase:
                 f"{interaction_call.import_prefix}.{interaction_call.name}",
             )
             and keyword_lists[tc_index]
+            and self.config.testCaseSplitPathRegEx
         )
 
+    def _create_rf_setup_call(self, setup_interaction: Union[AtomicInteractionCall, CompoundInteractionCall]) -> Setup:
+        cbr_parameters = self._create_cbr_parameters(setup_interaction)
+        if cbr_parameters:
+            logger.error("No variable assignment in [setup] possible.")
+        import_prefix = self._get_interaction_import_prefix(setup_interaction)
+        interaction_indent = self._get_interaction_indent(setup_interaction)
+        cbv_parameters = self._create_cbv_parameters(setup_interaction)
+        return Setup.from_params(
+            name=f"{import_prefix}{setup_interaction.name}",
+            args=tuple(cbv_parameters),
+            indent=interaction_indent,
+        )
+
+    def _create_rf_keyword_from_interaction_list(
+        self, keyword_name: str, interactions: list[InteractionCall]
+    ):
+        keyword_calls_lists = self._create_rf_keyword_calls(interactions)
+        keyword = Keyword(header=TestCaseName.from_params(keyword_name))
+        keyword.body.extend(keyword_calls_lists[0])
+        keyword.body.extend(LINE_SEPARATOR)
+        return keyword
+
+    def _create_rf_setup(self, setup_interactions: list[InteractionCall]) -> Union[Setup, None]:
+        rf_setup = None
+        if len(setup_interactions) == 1:
+            rf_setup = self._create_rf_setup_call(setup_interactions[0])
+        elif len(setup_interactions) > 1:
+            self.setup_keyword = self._create_rf_keyword_from_interaction_list(
+                f"Setup-{self.uid}", setup_interactions
+            )
+            rf_setup = Setup.from_params(name=self.setup_keyword.name)
+        return rf_setup
+
+    def _get_teardown_params(self, interaction_calls: list[InteractionCall]):
+        if len(interaction_calls) == 1:
+            interaction = interaction_calls[0]
+            return {
+                "name": f"{self._get_interaction_import_prefix(interaction)}{interaction.name}",
+                "args": tuple(self._create_cbv_parameters(interaction)),
+                "indent": self._get_interaction_indent(interaction),
+            }
+        return {"name": f"Teardown-{self.uid}"}
+
+    def _create_rf_teardown(self, teardown_interactions: list[InteractionCall]) -> Union[Setup, None]:
+        rf_teardown = None
+        if teardown_interactions:
+            teardown_params = self._get_teardown_params(teardown_interactions)
+            rf_teardown = Teardown.from_params(**teardown_params)
+            if len(teardown_interactions) > 1:
+                self.teardown_keyword = self._create_rf_keyword_from_interaction_list(
+                    f"Teardown-{self.uid}", teardown_interactions
+                )
+        return rf_teardown
+
     def to_robot_ast_test_cases(
         self,
-    ) -> List[
+    ) -> list[
         TestCase
     ]:  # TODO: Separate testcase splitting from this method --> new method: to_robot_ast_test_case
-        rf_keyword_lists = self._create_rf_keywords(self.interaction_calls)
-        rf_test_cases: List[TestCase] = []
-        multiple_tests = len(rf_keyword_lists) > 1
-        for index, rf_keywords in enumerate(rf_keyword_lists):
+        setup_interactions = list(
+            filter(
+                lambda interaction: interaction.sequence_phase == SequencePhase.Setup,
+                self.interaction_calls,
+            )
+        )
+        rf_setup = self._create_rf_setup(setup_interactions)
+        test_step_interactions = list(
+            filter(
+                lambda interaction: interaction.sequence_phase == SequencePhase.TestStep,
+                self.interaction_calls,
+            )
+        )
+        rf_keyword_call_lists = self._create_rf_keyword_calls(test_step_interactions)
+        teardown_interactions = list(
+            filter(
+                lambda interaction: interaction.sequence_phase == SequencePhase.Teardown,
+                self.interaction_calls,
+            )
+        )
+        rf_teardown = self._create_rf_teardown(teardown_interactions)
+        rf_test_cases: list[TestCase] = []
+        multiple_tests = len(rf_keyword_call_lists) > 1
+        for index, rf_keywords in enumerate(rf_keyword_call_lists):
             phase_pattern = self.config.phasePattern
             tc_name = (
                 phase_pattern.format(
-                    testcase=self.uid, index=index + 1, length=len(rf_keyword_lists)
+                    testcase=self.uid, index=index + 1, length=len(rf_keyword_call_lists)
                 )
                 if multiple_tests
                 else self.uid
@@ -229,43 +320,69 @@ class RfTestCase:
             # suffix = f' : Phase {index + 1}/{len(rf_keyword_lists)}' if multiple_tests else ''
             # tc_name = f"{self.uid}{suffix}"  # TODO later UID or Comments
             rf_test_case = TestCase(header=TestCaseName.from_params(tc_name))
-            rf_test_case.body.append(Tags.from_params(self.rf_tags))
+            if self.rf_tags:
+                rf_test_case.body.append(Tags.from_params(self.rf_tags))
+            if index == 0 and rf_setup:
+                rf_test_case.body.append(rf_setup)
             rf_test_case.body.extend(rf_keywords)
-            rf_test_case.body.extend(TESTCASE_SEPARATOR)
+            if index == len(rf_keyword_call_lists) - 1 and rf_teardown:
+                rf_test_case.body.append(rf_teardown)
+            if index != len(rf_keyword_call_lists) - 1:
+                rf_test_case.body.extend(LINE_SEPARATOR)
             rf_test_cases.append(rf_test_case)
         return rf_test_cases
 
     @staticmethod
-    def _create_cbv_parameters(interaction: AtomicInteractionCall) -> List[str]:
+    def _create_cbv_parameters(interaction: AtomicInteractionCall) -> list[str]:
         parameters = []
+        previous_arg_forces_named = False
         for name, value in interaction.cbv_parameters.items():
+            if value == "undef.":
+                previous_arg_forces_named = True
+                continue
             escaped_value = RfTestCase.escape_argument_value(value)
-            if name.endswith('='):
-                parameters.append(f"{name}{escaped_value}")
+            if re.match(r'^\*\*\ ?', name):
+                escaped_value = RfTestCase.escape_argument_value(value, False, False)
+                parameters.append(escaped_value)
+            elif re.match(r'^\*\ ?', name):
+                escaped_value = RfTestCase.escape_argument_value(value, False)
+                parameters.append(escaped_value)
+                previous_arg_forces_named = True
+            elif re.search(r'(^-\ ?|=$)', name) or previous_arg_forces_named:
+                pure_name = re.sub(r'(^-\ ?|=$)', "", name)
+                parameters.append(f"{pure_name}={escaped_value}")
+                previous_arg_forces_named = True
             else:
                 parameters.append(escaped_value)
         return parameters
 
     @staticmethod
-    def escape_argument_value(value: str) -> str:
-        value = re.sub(r'(?<!\\)=', r'\=', value)
-        value = re.sub(r'^#', r'\#', value)
-        value = re.sub(r'(?<= ) ', r'\ ', value)
-        return value
+    def escape_argument_value(value: str, space_escaping=True, equal_sign_escaping=True) -> str:
+        if space_escaping:
+            value = re.sub(r'^(?= )|(?<= )$|(?<= )(?= )', r'\\', value)
+        if equal_sign_escaping:
+            value = re.sub(r'(?<!\\)=', r'\=', value)
+        return re.sub(r'^#', r'\#', value)
 
     @staticmethod
-    def _create_cbr_parameters(interaction: AtomicInteractionCall) -> List[str]:
+    def _create_cbr_parameters(interaction: AtomicInteractionCall) -> list[str]:
         cbr_parameters = list(interaction.cbr_parameters.values())
         for index, parameter in enumerate(cbr_parameters):
             if not parameter.startswith('${'):
                 cbr_parameters[index] = f"${{{parameter}}}"
         return cbr_parameters
 
+    def _get_interaction_import_prefix(self, interaction: AtomicInteractionCall) -> str:
+        return (self.config.fullyQualified or False) * f"{interaction.import_prefix}."
+
+    def _get_interaction_indent(
+        self, interaction: Union[AtomicInteractionCall, CompoundInteractionCall]
+    ) -> str:
+        return SEPARATOR * interaction.indent if self.config.logCompoundInteractions else SEPARATOR
+
     def _create_rf_keyword(self, interaction: AtomicInteractionCall) -> KeywordCall:
-        import_prefix = (self.config.fullyQualified or False) * f"{interaction.import_prefix}."
-        interaction_indent = (
-            SEPARATOR * interaction.indent if self.config.logCompoundInteractions else SEPARATOR
-        )
+        import_prefix = self._get_interaction_import_prefix(interaction)
+        interaction_indent = self._get_interaction_indent(interaction)
         cbv_parameters = self._create_cbv_parameters(interaction)
         cbr_parameters = self._create_cbr_parameters(interaction)
         return KeywordCall.from_params(
@@ -296,14 +413,18 @@ class RfTestCase:
                 for param_name, param_value in interaction.cbv_parameters.items()
             ]
         )
+        cmd = []
         if cbr_params:
-            return f"#{SEPARATOR.join([cbr_params, interaction.name, cbv_params])}"
-        return f"#{SEPARATOR.join([interaction.name, cbv_params])}"
+            cmd.append(cbr_params)
+        cmd.append(interaction.name)
+        if cbv_params:
+            cmd.append(cbv_params)
+        return f"# {SEPARATOR.join(cmd)}"
 
     @staticmethod
     def _get_params_by_use_type(
         interaction: InteractionDetails, *param_use_types: ParameterUseType
-    ) -> Dict[str, str]:
+    ) -> dict[str, str]:
         return {
             parameter.name: parameter.value
             for parameter in interaction.parameters
@@ -312,10 +433,10 @@ class RfTestCase:
 
 
 def create_test_suites(
-    test_case_set_catalog: Dict[str, TestCaseSet],
+    test_case_set_catalog: dict[str, TestCaseSet],
     path_resolver: PathResolver,
     config: Configuration,
-) -> Dict[str, File]:
+) -> dict[str, File]:
     tcs_paths = path_resolver.tcs_paths
     test_suites = {}
     for uid, test_case_set in test_case_set_catalog.items():
@@ -335,12 +456,12 @@ class RobotInitFileBuilder:
         self, test_theme: TestStructureTreeNode, tt_path: PurePath, config: Configuration
     ) -> None:
         self.test_theme = test_theme
-        self.tt_path = tt_path
+        self.tt_path = PurePath(tt_path)
         self.config = config
 
     def create_init_file(self) -> File:
         sections = [self._create_setting_section()]
-        return File(sections, source=os.path.join(str(self.tt_path), "__init__"))
+        return File(sections, source=str(self.tt_path / "__init__"))
 
     def _create_setting_section(self) -> SettingSection:
         setting_section = SettingSection(header=SectionHeader.from_params(Token.SETTING_HEADER))
@@ -353,7 +474,7 @@ class RobotInitFileBuilder:
         )
         return setting_section
 
-    def _get_setting_section_metadata(self) -> Dict[str, str]:
+    def _get_setting_section_metadata(self) -> dict[str, str]:
         meta_data = {
             "UniqueID": self.test_theme.baseInformation.uniqueID,
             "Numbering": self.test_theme.baseInformation.numbering,
@@ -382,26 +503,45 @@ class RobotSuiteFileBuilder:
         self.test_case_set = test_case_set
         self.tcs_path = tcs_path
         self.config = config
-        self._rf_test_cases: List[RfTestCase] = [
+        self._rf_test_cases: list[RfTestCase] = [
             RfTestCase(test_case_details=test_case, config=config)
             for test_case in self.test_case_set.test_cases.values()
         ]
+        self.setup_keywords: list[Keyword] = []
+        self.teardown_keywords: list[Keyword] = []
 
     def create_test_suite_file(self) -> File:
         sections = [self._create_setting_section(), self._create_test_case_section()]
+        keyword_section = self._create_keywords_section()
+        if keyword_section:
+            sections[-1].body.extend(SECTION_SEPARATOR)
+            sections.append(keyword_section)
         return File(sections, source=str(self.tcs_path))
 
     def _create_test_case_section(self) -> TestCaseSection:
         test_case_section = TestCaseSection(header=SectionHeader.from_params(Token.TESTCASE_HEADER))
         robot_ast_test_cases = []
-        for test_case in self._rf_test_cases:
+        for index, test_case in enumerate(self._rf_test_cases):
             robot_ast_test_cases.extend(test_case.to_robot_ast_test_cases())
+            if index != len(self._rf_test_cases) - 1:
+                robot_ast_test_cases[-1].body.extend(LINE_SEPARATOR)
+            if test_case.setup_keyword:
+                self.setup_keywords.append(test_case.setup_keyword)
+            if test_case.teardown_keyword:
+                self.teardown_keywords.append(test_case.teardown_keyword)
         test_case_section.body.extend(robot_ast_test_cases)
-        test_case_section.body.extend(SECTION_SEPARATOR)
         return test_case_section
 
-    def _get_used_subdivisions(self) -> Dict[str, Set[str]]:
-        import_dict: Dict[str, Set[str]] = {}
+    def _create_keywords_section(self) -> Union[KeywordSection, None]:
+        if not self.setup_keywords and not self.teardown_keywords:
+            return None
+        keywords_section = KeywordSection(header=SectionHeader.from_params(Token.KEYWORD_HEADER))
+        keywords_section.body.extend(self.setup_keywords)
+        keywords_section.body.extend(self.teardown_keywords)
+        return keywords_section
+
+    def _get_used_subdivisions(self) -> dict[str, set[str]]:
+        import_dict: dict[str, set[str]] = {}
         for test_case in self._rf_test_cases:
             for root, import_names in test_case.used_imports.items():
                 if root not in import_dict:
@@ -410,13 +550,13 @@ class RobotSuiteFileBuilder:
                     import_dict[root].update(import_names)
         return import_dict
 
-    def _create_rf_variable_imports(self) -> List[VariablesImport]:
+    def _create_rf_variable_imports(self) -> list[VariablesImport]:
         return [
             VariablesImport.from_params(name=variable_file)
             for variable_file in self.config.forcedImport.variables
         ]
 
-    def _create_rf_resource_imports(self, import_dict: Dict[str, Set[str]]) -> List[ResourceImport]:
+    def _create_rf_resource_imports(self, import_dict: dict[str, set[str]]) -> list[ResourceImport]:
         resources = {
             str(resource)
             for resource_root, resources in import_dict.items()
@@ -430,31 +570,50 @@ class RobotSuiteFileBuilder:
         return [ResourceImport.from_params(res) for res in sorted(resource_paths)]
 
     def _create_resource_path(self, resource: str) -> str:
-        if (
-            not self.config.resourceDirectory
-        ):  # TODO Fehler: Hier springt er raus weil resourceDir nicht gesetzt, sollte aber mindestens das Mapping noch mitnehmen.
-            return f"{resource}.resource"
-        if not re.match(RELATIVE_RESOURCE_INDICATOR, self.config.resourceDirectory):
-            return f"{self.config.resourceDirectory}{ROBOT_PATH_SEPARATOR}{resource}.resource"
-        resource_dir = self._get_resource_directory()
-        resource_path = self.config.subdivisionsMapping.resources.get(resource)
-        resource_base_path = (
-            f"{('..' + ROBOT_PATH_SEPARATOR) * len(self.tcs_path.parts)}{resource_dir}"
-        )
-        if resource_path:
-            return f"{resource_base_path}{ROBOT_PATH_SEPARATOR}{resource_path}"
-        return f"{resource_base_path}{ROBOT_PATH_SEPARATOR}{resource}.resource"
-
-    def _get_resource_directory(self) -> str:
+        subdivision_mapping = self.config.subdivisionsMapping.resources.get(resource)
+        resource = re.sub(".resource", "", resource)
+        if not subdivision_mapping:
+            if not self.config.resourceDirectory:
+                return f"{resource}.resource"
+            if not re.match(RELATIVE_RESOURCE_INDICATOR, self.config.resourceDirectory):
+                return Path(self.config.resourceDirectory, f"{resource}.resource").as_posix()
+            generation_directory = self._replace_relative_resource_indicator(
+                self.config.generationDirectory
+            )
+            robot_file_path = Path(generation_directory) / self.tcs_path.parent
+            resource_directory = self._replace_relative_resource_indicator(
+                self.config.resourceDirectory
+            )
+            resource_import = (
+                Path(os.path.relpath(Path(resource_directory), robot_file_path))
+                / f"{resource}.resource"
+            )
+            return resource_import.as_posix()
         root_path = Path(os.curdir).absolute()
-        return os.path.relpath(
-            re.sub(
-                RELATIVE_RESOURCE_INDICATOR,
-                str(root_path).replace('\\', '\\\\'),
-                self.config.resourceDirectory,
-                flags=re.IGNORECASE,
-            ),
-            root_path,
+        subdivision_mapping = re.sub(
+            r"^{resourceDirectory}", self.config.resourceDirectory, subdivision_mapping
+        )
+        subdivision_mapping = re.sub(
+            RELATIVE_RESOURCE_INDICATOR, str(root_path).replace('\\', '/'), subdivision_mapping
+        )
+        return str(subdivision_mapping)
+
+    def _replace_relative_resource_indicator(self, path: Union[Path, str]) -> str:
+        root_path = Path(os.curdir).absolute()
+        return re.sub(
+            RELATIVE_RESOURCE_INDICATOR,
+            str(root_path).replace('\\', ROBOT_PATH_SEPARATOR),
+            str(path),
+            flags=re.IGNORECASE,
+        ).replace('\\', ROBOT_PATH_SEPARATOR)
+
+    def _get_relative_resource_directory(self) -> str:
+        root_path = Path(os.curdir).absolute()
+        return re.sub(
+            RELATIVE_RESOURCE_INDICATOR,
+            str(root_path).replace('\\', ROBOT_PATH_SEPARATOR),
+            self.config.resourceDirectory,
+            flags=re.IGNORECASE,
         ).replace('\\', ROBOT_PATH_SEPARATOR)
 
     @staticmethod
@@ -465,7 +624,7 @@ class RobotSuiteFileBuilder:
     def _is_resource(root_subdivision: str) -> bool:
         return root_subdivision == RESOURCE_IMPORT_TYPE
 
-    def _create_rf_library_imports(self, import_dict: Dict[str, Set[str]]) -> List[LibraryImport]:
+    def _create_rf_library_imports(self, import_dict: dict[str, set[str]]) -> list[LibraryImport]:
         libraries = {
             str(library)
             for library_root, libraries in import_dict.items()
@@ -478,7 +637,7 @@ class RobotSuiteFileBuilder:
         }
         return [LibraryImport.from_params(lib) for lib in sorted(lib_imports)]
 
-    def _create_rf_force_tags(self) -> Optional[ForceTags]:
+    def _create_rf_force_tags(self) -> Union[ForceTags, None]:
         tb_keyword_names = [keyword.name for keyword in self.test_case_set.details.spec.keywords]
         udfs = [udf.robot_tag for udf in self.test_case_set.details.spec.udfs if udf.robot_tag]
         force_tags = tb_keyword_names + udfs
@@ -486,7 +645,7 @@ class RobotSuiteFileBuilder:
             return ForceTags.from_params(force_tags)
         return None
 
-    def _create_rf_unknown_imports(self, import_dict: Dict[str, Set[str]]) -> List[Comment]:
+    def _create_rf_unknown_imports(self, import_dict: dict[str, set[str]]) -> list[Comment]:
         unknown_imports = {
             str(unknown_import)
             for root_subdivision, unknown_imports in import_dict.items()
